<?php

namespace FiloBlu\Flow\Model\Actions\From;

/**
 * Class AbstractAddLink
 * @package FiloBlu\Flow\Model\Actions\From
 */
abstract class AbstractAddLink
{

    /**
     * @var \Magento\Catalog\Api\Data\ProductLinkInterfaceFactory
     */
    protected $productLinkInterfaceFactory;

    /**
     * @var \Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\ConfigurableFactory
     */
    protected $configurableFactory;

    /**
     * @var \Magento\Framework\App\ResourceConnection 
     */
    protected $connection;

    /**
     * @var \Magento\Framework\App\ProductMetadataInterface
     */
    protected $productMetadata;

    /**
     * Addcrosssell constructor.
     * @param \Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\ConfigurableFactory $configurableFactory
     * @param \Magento\Framework\App\ResourceConnection $connection
     */
    public function __construct(
        \Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\ConfigurableFactory $configurableFactory,
        \Magento\Framework\App\ResourceConnection $connection
    )
    {
        $this->configurableFactory = $configurableFactory;
        $this->connection = $connection;
    }

    /**
     * @return int of \Magento\Catalog\Model\Product\Link::LINK_TYPE_RELATED, \Magento\Catalog\Model\Product\Link::LINK_TYPE_CROSSSELL, \Magento\Catalog\Model\Product\Link::LINK_TYPE_UPSELL
     */
    abstract protected function getLinkType();

    /**
     * @param $product
     * @param $value
     * @param $storeId
     * @throws \Exception
     */
    public function process($product, $value, $storeId)
    {
        if (empty($value)) {
            return;
        }

        if ($storeId != 0) {
            throw new \Exception("The function is avalaible only for the ADMIN store.");
        }

        $products  =  $this->getTargetProducts($product);
        $productsToLink  = $this->getProductsToLink($value);

        foreach($products as $productId)
        {
            foreach ($productsToLink as $toLink) {
                $this->setLink($productId, $toLink['entity_id'],$this->getLinkType());
            }
        }

    }

    /**
     * Create or update link
     *
     * @param $product_id int In Magento Enterprise Edition $product_id references catalog_product_entity.row_id
     * @param $linked_product_id int references catalog_product_entity.entity_id
     * @param $link_type_id int one of \Magento\Catalog\Model\Product\Link::LINK_TYPE_RELATED, \Magento\Catalog\Model\Product\Link::LINK_TYPE_CROSSSELL, \Magento\Catalog\Model\Product\Link::LINK_TYPE_UPSELL
     *
     * @return int
     */
    protected function setLink($product_id, $linked_product_id, $link_type_id)
    {
        $table = $this->connection->getTableName('catalog_product_link');
        return $this->connection->getConnection()->insertOnDuplicate($table, [
            'product_id' => $product_id,
            'linked_product_id' => $linked_product_id,
            'link_type_id' => $link_type_id
        ]);
    }

    /**
     * @param $product
     * @return array
     */
    protected function getTargetProducts($product)
    {
        $products = [$product->getId()];

        if ($product->getTypeId() == \Magento\Catalog\Model\Product\Type::DEFAULT_TYPE) {
            $parentIds = $this->configurableFactory->create()->getParentIdsByChild($product->getId());
            return array_merge($products, $parentIds);
        }

        return $products;
    }

    /**
     * @param $value
     * @return array
     */
    protected function getProductsToLink($value)
    {
        $toLink = explode('|', $value);
        $table = $this->connection->getTableName('catalog_product_entity');
        $sql = $this->connection->getConnection()->select()->from($table, ['sku', 'entity_id', 'row_id'])->where('sku IN (?)', $toLink)->assemble();

        return $this->connection->getConnection()->fetchAll($sql, \Zend_Db::FETCH_ASSOC);
    }

}
