<?php

namespace FiloBlu\Flow\Model\Actions\From;

/**
 * Class Addincrementstock
 *
 * @package FiloBlu\Flow\Model\Actions\From
 */
class Addincrementstock
{
    /**
     * @var \Magento\CatalogInventory\Api\StockRegistryInterface
     */
    protected $stockRegistry;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * Addincrementstock constructor.
     * @param \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     */
    public function __construct(
        \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry,
        \Magento\Store\Model\StoreManagerInterface $storeManager
    )
    {
        $this->stockRegistry = $stockRegistry;
        $this->storeManager = $storeManager;
    }

    /**
     * @param $product
     * @param $value
     * @param $storeId
     * @throws \Exception
     */
    public function process($product, $value, $storeId)
    {
        if ($storeId != 0) {
            throw new \Exception('The function is avalaible only for the ADMIN store.');
        }

        $this->storeManager->setCurrentStore($storeId);

        $productId = $product->getId();

        if (empty($productId)) {
            throw new \Exception(sprintf('Product not found! product_code: \'%s\'', $product->getSku()));
        }

        if ($value != null && $value != '') {
            throw new \Exception("Invalid value supplied '{$value}'");
        }
        $stock = $this->stockRegistry->getStockItem($productId);

        if (is_numeric($value) && $value > 0) {
            $stock->setUseConfigEnableQtyInc(false);
            $stock->setUseConfigQtyIncrements(false);
            $stock->setEnableQtyIncrements(true);
            $stock->setQtyIncrements($value);
            $stock->save();
        } else if (is_numeric($value) && $value < 0) {
            $stock->setEnableQtyIncrements(false);
            $stock->setUseConfigEnableQtyInc(true);
            $stock->setUseConfigQtyIncrements(true);
            $stock->save();
        } else {
            throw new \Exception("Product increment value wrong: '{$value}'");
        }

    }

}
