<?php

namespace FiloBlu\Flow\Model\Channel;

/**
 * Class Config
 * @package FiloBlu\Flow\Model\Channel
 */
class Config
{
    /**
     * @var bool
     */
    public $initialized = false;

    /**
     * @var string
     */
    public $file = 'map.json';

    /**
     * @var array
     */
    protected $_mappedData = [];

    /**
     * @var \Monolog\Logger
     */
    protected $_logger;

    /**
     * @var \FiloBlu\Flow\Model\Channel
     */
    protected $_channel_model;

    /**
     * Config constructor.
     * @param \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
     * @param \FiloBlu\Flow\Model\Channel $channelModel
     */
    public function __construct(
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \FiloBlu\Flow\Model\Channel $channelModel
    )
    {
        $this->_logger = $loggerProvider->getLogger();
        $this->_channel_model = $channelModel;
    }

    /**
     * @param $name
     * @return $this
     */
    public function load($name)
    {

        // $name -> DB flow_channels.name
        // unique value (string)

        $this->_channel_model->load($name, 'name');

        if ($this->_channel_model) {
            try {
                // TODO: really necessary?
                $json_content = str_replace(["\n", "\t", "\r"], '', $this->_channel_model->getData('data'));

                $mapped_config = json_decode($json_content, true);
                //$mapped_config->map = null;
                $this->_mappedData = $mapped_config;
                $this->initialized = true;
                $this->_logger->info(__METHOD__ . "Config " . $name . " - Loaded map json file");
            } catch (\Exception $e) {
                $this->_logger->info(__METHOD__ . "Config " . $name . " - Fail to load map json file");
            }
        } else {
            $this->_logger->info(__METHOD__ . "Config " . $name . " - Model for channel data not exist for json map");
        }

        return $this;

    }

    /**
     * @return array
     */
    public function getConfigData()
    {
        return $this->_mappedData;
    }

    /**
     * @param $field
     * @return mixed|null
     */
    public function get($field)
    {
        if (isset($this->_mappedData[$field])) {
            return $this->_mappedData[$field];
        }
        return null;
    }

    /**
     * @return array
     */
    public function getFields()
    {
        return array_keys($this->_mappedData);
    }

    /**
     * @param $data
     * @return bool
     */
    public function checkData($data)
    {
        if (count($data) == count($this->_mappedData)) {
            return true;
        }
        return false;
    }

    /**
     * @return bool
     */
    public function usesMap()
    {
        if (isset($this->_mappedData['use_map'])) {
            return boolval($this->_mappedData['use_map']);
        }

        return false;
    }
}
